<?php

namespace FiloBlu\Storelocator\Helper;

use FiloBlu\Storelocator\Model\Rma\Returntostore;
use FiloBlu\Storelocator\Model\TagsFactory;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Asset\Repository;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Store\Model\Information;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

class Data extends AbstractHelper
{
    const CONFIG_IMAGE_DIR_PREFIX = 'filoblu/storelocator/';
    const STORES_IMAGE_DIR_PREFIX = 'filoblu/storelocator/stores/image/';
    //const USE_STANDARD_PICKRMA_MAIL_FLOW = 'carriers/returninstore/pickrma_use_standard_mail_flow';
    //const PICKRMA_CUSTOMER_CARE_MAIL_TEMPLATE = 'carriers/returninstore/pickrma_customercare_mail_template';
    //const PICKRMA_USE_TEST_MAIL_RECIPIENTS = 'carriers/returninstore/pickrma_use_test_email_recipients';
    //const PICKRMA_CUSTOMERCARE_TEST_MAIL_RECIPIENT = 'carriers/returninstore/pickrma_customercare_test_email_recipient';
    //const PICKRMA_STORE_TEST_MAIL_RECIPIENT = 'carriers/returninstore/pickrma_store_test_email_recipient';
    //const PICKRMA_NON_STANDARD_FLOW_MAIL_SETTINGS = 'sales_email/magento_rma/copy_to';

    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var Returntostore
     */
    protected $_returntostore;
    /**
     * @var TagsFactory
     */
    private $_tagsFactory;
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;
    /**
     * @var \Magento\Directory\Model\RegionFactory
     */
    protected $regionFactory;
    /**
     * @var \FiloBlu\Storelocator\Model\StoresFactory
     */
    protected $storesFactory;

    public function __construct(
        Context $context,
        StoreManagerInterface $storeManager,
        Repository $assetRepo,
        Returntostore $_returntostore,
        TagsFactory $_tagsFactory,
        OrderRepositoryInterface $orderRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Directory\Model\RegionFactory $regionFactory,
        \FiloBlu\Storelocator\Model\StoresFactory $storesFactory
    )
    {
        $this->_storeManager = $storeManager;
        $this->scopeConfig = $context->getScopeConfig();
        $this->assetRepo = $assetRepo;
        $this->request = $context->getRequest();
        $this->urlBuilder = $context->getUrlBuilder();
        $this->_returntostore = $_returntostore;
        $this->_tagsFactory = $_tagsFactory;
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->regionFactory = $regionFactory;
        $this->storesFactory = $storesFactory;
    }

    public function geocode($address)
    {
        // url encode the address
        $address = urlencode($address);

        // google map geocode api url
        $key = $this->getConfig('filoblu_storelocator_section/general/gmaps_api_key_server_side');
        $url = $this->getConfig('filoblu_storelocator_section/geocode/gmaps_geocode_api_url') . "?address={$address}&key={$key}";

        // get the json response
        $resp_json = file_get_contents($url);

        // decode the json
        $resp = json_decode($resp_json, true);

        // response status will be 'OK', if able to geocode given address
        if ($resp['status'] == 'OK') {

            // get the important data
            $lati = $resp['results'][0]['geometry']['location']['lat'];
            $longi = $resp['results'][0]['geometry']['location']['lng'];
            $formatted_address = $resp['results'][0]['formatted_address'];

            // verify if data is complete
            if ($lati && $longi && $formatted_address) {

                // put the data in the array
                $data_arr = array();

                array_push(
                    $data_arr,
                    $lati,
                    $longi,
                    $formatted_address
                );

                return $data_arr;

            } else {
                return false;
            }

        } else {
            return false;
        }
    }

    public function getConfig($config_path)
    {
        return $this->scopeConfig->getValue(
            $config_path,
            ScopeInterface::SCOPE_STORE
        );
    }

    public function getBaseUrlConfigImage()
    {
        return $this->_storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_MEDIA) . SELF::CONFIG_IMAGE_DIR_PREFIX;
    }

    public function getBaseUrlStoresImage()
    {
        return $this->_storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_MEDIA) . SELF::STORES_IMAGE_DIR_PREFIX;
    }

    /**
     * @return mixed by erry
     */
    public function getBaseUrl()
    {
        return $this->_storeManager->getStore()->getBaseUrl();
    }

    /**
     * @param $fileId
     * @param array $params
     * @return mixed by erry
     */
    public function getViewFileUrl($fileId, array $params = [])
    {
        try {
            $params = array_merge(['_secure' => $this->request->isSecure()], $params);
            return $this->assetRepo->getUrlWithParams($fileId, $params);
        } catch (LocalizedException $e) {
            return $this->urlBuilder->getUrl('', ['_direct' => 'core/index/notFound']);
        }
    }

    /**
     * @param null $orderId
     * @return array
     */
    public function getTagFilterHandle($orderId = null)
    {
        // READ TAGS
        $alltags = $this->_tagsFactory->create()->getAllAvailableTags();
        $tags = array();

        if (count($alltags) > 0) {
            foreach ($alltags as $tag) {
                $tags[$tag->getName()] = $tag->getId();
            }
        }

        if (!is_null($orderId) &&
            $this->getTagFiltersEnabled() == 1 &&
            is_array($this->getTagFiltersLabels()) &&
            is_array($this->getTagCategoryFilterIds())) {

            $categories = $this->_returntostore->getItemsCategoriesByOrderId($orderId);
            $countCat = count($categories);
            $setting = array();
            $flatted_categories = array();
            $counter = 0;
            array_walk_recursive($categories, function ($a) use (&$flatted_categories) {
                $flatted_categories[] = $a;
            }); // flat multidimensional array
            $categories = $flatted_categories;
            $tag_setting = $this->getTagFiltersLabels();
            if (is_array($this->getTagCategoryFilterIds())) {
                $setting = $this->getTagCategoryFilterIds();
            }
            foreach ($categories as $cat) {
                if (in_array($cat, $setting)) { // $cat = all my categories  $setting = my outlet ids
                    $counter++;
                }
            }

            if ($counter > 0) {
                // outlet result -> consider Storelocator Setting tag ids

                foreach ($tags as $key => $tag_ids) {
                    if (is_array($tag_setting) && !in_array($key, $tag_setting)) {
                        unset($tags[$key]);
                    }
                }
            } else {
                // monobrand result -> consider tag ids excluded
                foreach ($tags as $key => $tag_ids) {
                    if (is_array($tag_setting) && in_array($key, $tag_setting)) {
                        unset($tags[$key]);
                    }
                }
            }

        }
        return $tags;
    }

    /**
     * @return mixed by erry
     */
    public function getTagFiltersEnabled()
    {
        return $this->scopeConfig->getValue('filoblu_storelocator_section/return_to_store/enable_tag_filter_by_category', ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return false|string[]
     */
    public function getTagFiltersLabels()
    {
        $params = false;
        $enabled = $this->getTagFiltersEnabled();
        $return = false;
        if ($enabled == '1') {
            $params = $this->scopeConfig->getValue('filoblu_storelocator_section/return_to_store/tag_filter_by_category', ScopeInterface::SCOPE_STORE);
            $params = unserialize($params);
            foreach ($params as $label) {
                if (array_key_exists('tag_filter_label', $label)) {
                    $return = $label['tag_filter_label'];
                    $return = explode(',', $return);
                }
            }
        }
        return $return;
    }

    /**
     * @return false|string[]
     */
    public function getTagCategoryFilterIds()
    {
        $params = false;
        $enabled = $this->scopeConfig->getValue('filoblu_storelocator_section/return_to_store/enable_tag_filter_by_category', ScopeInterface::SCOPE_STORE);
        $return = false;
        if ($enabled == '1') {
            $params = $this->scopeConfig->getValue('filoblu_storelocator_section/return_to_store/tag_filter_by_category', ScopeInterface::SCOPE_STORE);
            $params = unserialize($params);
            foreach ($params as $ids) {
                if (array_key_exists('tag_category_filter_id', $ids)) {
                    $return = $ids['tag_category_filter_id'];
                    $return = explode(',', $return);
                }
            }
        }
        return $return;
    }

    public function getItemsCategoriesByOrderId($orderId = null)
    {
        return $this->_returntostore->getItemsCategoriesByOrderId($orderId);
    }

    /**
     * @param $hash
     * @return OrderInterface
     * @throws NoSuchEntityException
     */
    public function getOrderByHash($hash)
    {
        $searchCriteria = $this->searchCriteriaBuilder
            ->addFilter('pickup_in_store_hash', $hash)
            ->addFilter('shipping_method','pickupinstore_pickupinstore')
            ->create();

        $result = $this->orderRepository->getList($searchCriteria)->getItems();
        $order =  array_shift($result);
        if (!$order){
            throw NoSuchEntityException::singleField('hash',$hash);
        }

        return $order;
    }

    /**
     * @param OrderInterface $order
     */
    public function saveOrder(OrderInterface $order)
    {
        $this->orderRepository->save($order);
    }

    public function getStoreDetails($storeId) {
        $store = $this->storesFactory->create()->load($storeId);

        $data = [
            'city' => $store->getCity(),
            'countryId' => null,
            'postcode' => $store->getPostcode(),
            'region_id' => $store->getRegionId(),
            'street1'=> $store->getStreet(),
            'street2' => null,
            'firstname' => $this->getConfig(\Magento\Rma\Model\Shipping::XML_PATH_CONTACT_NAME),
            'country'=> $store->getCountry(),
            'region' => $store->getRegion(),
            'company' => $store->getTitle(),
            'telephone' => $this->getConfig(Information::XML_PATH_STORE_INFO_PHONE),
            'email' => null,// Information::XML_PATH_STORE_INFO_PHONE,
        ];

        return $data;
    }

    public function getPickupInStoreEnabled() {
        return $this->getConfig("carriers/pickupinstore/active");
    }

    public function getReturnToStoreEnabled() {
        return $this->scopeConfig->getValue('carriers/returninstore/pickrma_enable', ScopeInterface::SCOPE_STORE);
    }

    public function getUseStandardRmaMailFlow() {
        return $this->getConfig('carriers/returninstore/pickrma_use_standard_mail_flow');
    }

    public function getRmaMailCopyTo() {
        return $this->getConfig('sales_email/magento_rma/copy_to');
    }

    public function getReturnInStoreEmailTemplate() {
        return $this->getConfig('carriers/returninstore/pickrma_customercare_mail_template');
    }

    public function getUseTestEmailRecipients() {
        return $this->getConfig('carriers/returninstore/pickrma_use_test_email_recipients');
    }

    public function getCustomercareTestEmail() {
        return $this->getConfig('carriers/returninstore/pickrma_customercare_test_email_recipient');
    }

    public function getStoreTestEmail() {
        return $this->getConfig('carriers/returninstore/pickrma_store_test_email_recipient');
    }

    public function getCustomercareEmail() {
        return $this->getConfig('carriers/returninstore/pickrma_customercare_email_recipient');
    }
}
