<?php

namespace FiloBlu\Storelocator\Helper;

use FiloBlu\Storelocator\Model\Import\Stores as ImportModel;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Sales\Api\OrderRepositoryInterface;

class Import extends \FiloBlu\Storelocator\Helper\Data
{
    protected $_storesFactory;
    protected $_tagsFactory;
    protected $_workingTimesFactory;
    protected $_regionFactory;

    /**
     * Import constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\View\Asset\Repository $assetRepo
     * @param \FiloBlu\Storelocator\Model\StoresFactory $storesFactory
     * @param \FiloBlu\Storelocator\Model\TagsFactory $tagsFactory
     * @param \FiloBlu\Storelocator\Model\WorkingTimesFactory $workingTimesFactory
     * @param \Magento\Directory\Model\RegionFactory $regionFactory
     * @param \FiloBlu\Storelocator\Model\Rma\Returntostore $returntostore
     * @param OrderRepositoryInterface $orderRepository
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\View\Asset\Repository $assetRepo,
        \FiloBlu\Storelocator\Model\StoresFactory $storesFactory,
        \FiloBlu\Storelocator\Model\TagsFactory $tagsFactory,
        \FiloBlu\Storelocator\Model\WorkingTimesFactory $workingTimesFactory,
        \Magento\Directory\Model\RegionFactory $regionFactory,
        \FiloBlu\Storelocator\Model\Rma\Returntostore $returntostore,
        OrderRepositoryInterface $orderRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder
    )
    {
        $this->_storesFactory = $storesFactory;
        $this->_tagsFactory = $tagsFactory;
        $this->_workingTimesFactory = $workingTimesFactory;
        $this->_regionFactory = $regionFactory;

        parent::__construct($context, $storeManager, $assetRepo, $returntostore, $tagsFactory, $orderRepository, $searchCriteriaBuilder, $regionFactory, $storesFactory);
    }

    public function importDbData($data)
    {
        if ($data) {
            foreach ($data as $storeData) {
                if (!$storeData[ImportModel::LATITUDE] || !$storeData[ImportModel::LONGITUDE]) {
                    $address = (isset($storeData[ImportModel::STREET]) ? $storeData[ImportModel::STREET] . " " : "") .
                        (isset($storeData[ImportModel::POSTCODE]) ? $storeData[ImportModel::POSTCODE] . " " : "") .
                        (isset($storeData[ImportModel::CITY]) ? $storeData[ImportModel::CITY] . " " : "") .
                        (isset($storeData[ImportModel::COUNTRY]) ? $storeData[ImportModel::COUNTRY] . " " : "");
                    $geocodeData = $this->geocode($address);
                    if ($geocodeData) {
                        $storeData[ImportModel::LATITUDE] = $geocodeData[0];
                        $storeData[ImportModel::LONGITUDE] = $geocodeData[1];
                    }
                }

                if (!$storeData[ImportModel::ENABLED] && $storeData[ImportModel::ENABLED] != 0) {
                    $storeData[ImportModel::ENABLED] = 1;
                }

                if (!$storeData[ImportModel::IS_ENABLED_FOR_PICKUP_IN_STORE] && $storeData[ImportModel::IS_ENABLED_FOR_PICKUP_IN_STORE] != 0) {
                    $storeData[ImportModel::IS_ENABLED_FOR_PICKUP_IN_STORE] = 1;
                }

                if (!$storeData[ImportModel::IS_ENABLED_SHOP_DETAILS] && $storeData[ImportModel::IS_ENABLED_SHOP_DETAILS] != 0) {
                    $storeData[ImportModel::IS_ENABLED_SHOP_DETAILS] = 1;
                }

                if (!$storeData[ImportModel::STORE_CODE] && $storeData[ImportModel::STORE_CODE] != '') {
                    $storeData[ImportModel::STORE_CODE] = $storeData[ImportModel::STORE_CODE];
                }

                if (!$storeData[ImportModel::STORE_IDS]) {
                    $storeData[ImportModel::STORE_IDS] = 0;
                }

                $storeData[ImportModel::EMAIL] = trim($storeData[ImportModel::EMAIL]);

                if (isset($storeData[ImportModel::REGION_CODE]) && $storeData[ImportModel::REGION_CODE] && $storeData[ImportModel::COUNTRY]) {
                    $region = $this->_regionFactory->create();
                    $regionId = $region->loadByCode($storeData[ImportModel::REGION_CODE], $storeData[ImportModel::COUNTRY])->getId();
                    $storeData["region_id"] = $regionId;
                }

                $storeModel = $this->_storesFactory->create();
                $storeModel->addData($storeData);

                // Tags
                if (isset($storeData[ImportModel::TAGS])) {
                    $tagsArray = [];
                    $tags = explode(",", $storeData[ImportModel::TAGS]);

                    foreach ($tags as $tag) {
                        // check if exists
                        $modelTag = $this->_tagsFactory->create()->loadByTagName($tag);

                        if ($modelTag->getId()) {
                            $tagsArray[] = $modelTag->getId();
                        } else {
                            $modelTag->setName($tag);
                            $modelTag->setEnabled(1);
                            $tagsArray[] = $modelTag->save()->getId();
                        }
                    }

                    $storeModel->setTagsData($tagsArray);
                }

                // Working Times
                $objWorkingTimes = [];
                for ($wt = 1; $wt <= 7; $wt++) {
                    $dayStr = '';

                    switch ($wt) {
                        case 1:
                            $dayStr = "monday";
                            break;
                        case 2:
                            $dayStr = "tuesday";
                            break;
                        case 3:
                            $dayStr = "wednesday";
                            break;
                        case 4:
                            $dayStr = "thursday";
                            break;
                        case 5:
                            $dayStr = "friday";
                            break;
                        case 6:
                            $dayStr = "saturday";
                            break;
                        case 7:
                            $dayStr = "sunday";
                            break;

                        default:
                            break;
                    }

                    $currentDay = "wt_" . $dayStr;

                    if (isset($storeData[$currentDay]) && !empty($storeData[$currentDay])) {
                        $storeData[$currentDay] = trim($storeData[$currentDay]);

                        if (strpos($storeData[$currentDay], ",") === false) {
                            // Continued timetable

                            $time = explode("-", $storeData[$currentDay]);

                            $objWorkingTimes[$dayStr . '_morning_time_from'] = '01/01/2000 ' . $time[0] . ':00';
                            $objWorkingTimes[$dayStr . '_morning_time_to'] = '01/01/2000 00:00:00';
                            $objWorkingTimes['closed_' . $dayStr . '_morning'] = 0;

                            $objWorkingTimes[$dayStr . '_afternoon_time_from'] = '01/01/2000 00:00:00';
                            $objWorkingTimes[$dayStr . '_afternoon_time_to'] = '01/01/2000 ' . $time[1] . ':00';
                            $objWorkingTimes['closed_' . $dayStr . '_afternoon'] = 0;
                        } else {
                            // Hours range
                            $morningTime = $this->getTimeMorning($storeData[$currentDay]);
                            $afternoonTime = $this->getTimeAfternoon($storeData[$currentDay]);

                            if ($morningTime) {
                                $time = explode("-", $morningTime);

                                $objWorkingTimes[$dayStr . '_morning_time_from'] = '01/01/2000 ' . $time[0] . ':00';
                                $objWorkingTimes[$dayStr . '_morning_time_to'] = '01/01/2000 ' . $time[1] . ':00';
                                $objWorkingTimes['closed_' . $dayStr . '_morning'] = 0;
                            } else {
                                $objWorkingTimes[$dayStr . '_morning_time_from'] = '01/01/2000 00:00:00';
                                $objWorkingTimes[$dayStr . '_morning_time_to'] = '01/01/2000 00:00:00';
                                $objWorkingTimes['closed_' . $dayStr . '_morning'] = 1;
                            }

                            if ($afternoonTime) {
                                $time = explode("-", $afternoonTime);

                                $objWorkingTimes[$dayStr . '_afternoon_time_from'] = '01/01/2000 ' . $time[0] . ':00';
                                $objWorkingTimes[$dayStr . '_afternoon_time_to'] = '01/01/2000 ' . $time[1] . ':00';
                                $objWorkingTimes['closed_' . $dayStr . '_afternoon'] = 0;
                            } else {
                                $objWorkingTimes[$dayStr . '_afternoon_time_from'] = '01/01/2000 00:00:00';
                                $objWorkingTimes[$dayStr . '_afternoon_time_to'] = '01/01/2000 00:00:00';
                                $objWorkingTimes['closed_' . $dayStr . '_afternoon'] = 1;
                            }
                        }
                    } else {
                        $objWorkingTimes[$dayStr . '_morning_time_from'] = '01/01/2000 00:00:00';
                        $objWorkingTimes[$dayStr . '_morning_time_to'] = '01/01/2000 00:00:00';
                        $objWorkingTimes['closed_' . $dayStr . '_morning'] = 0;

                        $objWorkingTimes[$dayStr . '_afternoon_time_from'] = '01/01/2000 00:00:00';
                        $objWorkingTimes[$dayStr . '_afternoon_time_to'] = '01/01/2000 00:00:00';
                        $objWorkingTimes['closed_' . $dayStr . '_afternoon'] = 0;
                    }
                }

                $storeModel->setObjWorkingTimes($objWorkingTimes);
                $storeModel->save();
            }
        }
    }

    public function validateHours($hours)
    {
        if (!empty($hours)) {
            $hours = trim($hours);

            if (strpos($hours, ",") === false) {
                // Check if continued timetable

                if (!$this->validateTimeRange($hours)) {
                    return false;
                } else {
                    return true;
                }
            } else {
                // Hours range
                $morningTime = $this->getTimeMorning($hours);
                $afternoonTime = $this->getTimeAfternoon($hours);

                if (!$morningTime && !$afternoonTime) {
                    return false;
                }

                if ($morningTime) {
                    if (!$this->validateTimeRange($morningTime)) {
                        return false;
                    }
                }

                if ($afternoonTime) {
                    if (!$this->validateTimeRange($afternoonTime)) {
                        return false;
                    }
                }

                return true;
            }
        } else {
            return true;
        }
    }

    private function validateTimeRange($timeRange)
    {
        $time = explode("-", $timeRange);

        if (isset($time[0])) {
            if (!$this->validateTimeRegEx($time[0])) {
                return false;
            }
        } else {
            return false;
        }

        if (isset($time[1])) {
            if (!$this->validateTimeRegEx($time[1])) {
                return false;
            }
        } else {
            return false;
        }

        return true;
    }

    private function validateTimeRegEx($time)
    {
        return preg_match("#^([01]?[0-9]|2[0-3]):[0-5][0-9](:[0-5][0-9])?$#", $time);
    }

    private function getTimeMorning($hours)
    {
        $arrayHours = explode(",", $hours);

        if (count($arrayHours) > 2) {
            return false;
        }

        if (isset($arrayHours[0])) {
            return $arrayHours[0];
        }

        return false;
    }

    private function getTimeAfternoon($hours)
    {
        $arrayHours = explode(",", $hours);

        if (count($arrayHours) > 2) {
            return false;
        }

        if (isset($arrayHours[1])) {
            return $arrayHours[1];
        }

        return false;
    }

    public function validateEnabled($enabled)
    {
        if ($enabled != 0 && $enabled != 1) {
            return false;
        } else {
            return true;
        }
    }

    public function validateStoreIds($storeIds)
    {
        $isValid = true;

        $storeIdsArray = explode(",", $storeIds);

        foreach ($storeIdsArray as $storeId) {
            if (!isset($this->_storeManager->getStores(false, false)[trim($storeId)])) {
                $isValid = false;
            }
        }

        return $isValid;
    }
}
